@@ -3,20 +3,29 @@ Created on 2021-04-15
 
 @author: wf
 '''
-from ormigrate.fixer import PageFixer
-from ormigrate.rating import Rating
+from smw.pagefixer import PageFixer,PageFixerManager
+from smw.rating import Rating,RatingType
 from openresearch.openresearch import OpenResearch
 import yaml
 import os
+from smw.pagefixer import PageFixerManager
+from ormigrate.fixer import ORFixer
 
-class CurationQualityChecker(PageFixer):
+
+class CurationQualityChecker(ORFixer):
     '''
-    https://github.com/SmartDataAnalytics/OpenResearch/issues/170
-        
-    Curation quality check
+    see purpose and issue
     '''
+    purpose="Curation quality check"
+    issue="https://github.com/SmartDataAnalytics/OpenResearch/issues/170"
     userrating={}
     
+    def __init__(self,pageFixerManager):
+        '''
+        Constructor
+        '''
+        super(CurationQualityChecker, self).__init__(pageFixerManager)
+    
     @classmethod
     def loadUserRating(cls,path=None):
         '''
@@ -38,8 +47,11 @@ class CurationQualityChecker(PageFixer):
             userName=entityRecord['lastEditor'].replace('User:','')
             if userName in userRating:
                 painRecord=userRating[userName]
-                return Rating(painRecord['pain'],Rating.ok,painRecord['hint'])
+                return Rating(painRecord['pain'],RatingType.ok,painRecord['hint'])
             else:
-                return Rating(7,Rating.invalid,'last edited by unrated curator')
+                return Rating(7,RatingType.invalid,'last edited by unrated curator')
         else:   
-            return Rating(10,Rating.missing,'bug: lastEditor not set')
\ No newline at end of file
+            return Rating(10,RatingType.missing,'bug: lastEditor not set')
+        
+if __name__ == "__main__":
+    PageFixerManager.runCmdLine([CurationQualityChecker])
\ No newline at end of file
