@@ -1,59 +1,42 @@
-from openresearch.event import Event
-from ormigrate.fixer import PageFixer
 from geograpy.locator import LocationContext, Location, City, Country, Region
+from smw.pagefixer import PageFixerManager
+from ormigrate.fixer import ORFixer
+from smw.rating import Rating, RatingType, EntityRating
 
-class LocationFixer(PageFixer):
-    '''
-    fixes Locations
-    '''
-    COUNTRY = "Country"
-    REGION = "State"
-    CITY = "City"
 
-    def __init__(self, wikiClient, debug=False):
+class LocationFixer(ORFixer):
+    '''
+    see purpose and issue
+    '''
+    purpose="fixes Locations"
+    issue="https://github.com/SmartDataAnalytics/OpenResearch/issues/220"
+    
+    # value of type attribute of Locations to be fixed
+    COUNTRY = "country"
+    REGION = "region"
+    CITY = "city"
+
+    def __init__(self,pageFixerManager):
         '''
         Constructor
         '''
-        # call super constructor
-        super(LocationFixer, self).__init__(wikiClient)
-        self.debug = debug
-        #self.locationCorpus=LocationCorpus()
-        self.locationContext=LocationContext.fromJSONBackup()
+        super(LocationFixer, self).__init__(pageFixerManager)
+        LocationFixer.locationContext=self.getORLocationContext()
 
-    def fixEventRecords(self, events:list):
-        """
-        Gets list of dicts (list of events) and tries to fix the location entries
-        """
-        count=0
-        stats={}
-        for event_unfixed in events:
-            event, errors = self.fixEventRecord(event_unfixed)
-            print(errors)
-            if errors is not None:
-                for error in errors.keys():
-                    if error in stats:
-                        stats[error]+=1
-                    else:
-                        stats[error]=1
-        print(stats)
-
-    def fixEvents(self, events:list):
+    def fix(self,rating:EntityRating):
         '''
-        fixes the location of the given events
+        tries fixing the location entries of the  given entity
+        '''
+        eventRecord = rating.getRecord()
+        self.fixEventRecord(eventRecord)
 
+    def fixEventRecord(self, event:dict, errors:dict=None, bestFit=True):
+        '''
         Args:
-            events(list): list of Event objects that should be fixed
+            event(dict): event records containing the location values that should be fixed
+            errors(dict): dictonary containing the errors of the given event record → new errors are added to the dict
+            bestFit(bool): If true the best/closed fit for a location is chosen (e.g. city with highest population). Otherwise a fix is only applied if the location can be identified with certainty.
         '''
-        for event in events:
-            self.fixEvent(event)
-
-    def fixEvent(self, event:Event):
-        '''
-        fixes the location of the given event
-        '''
-        self.fixEventRecord(event.__dict__)
-
-    def fixEventRecord(self, event:dict, errors=None):
         # event location values
         if errors is None:
             errors = {}
@@ -111,21 +94,22 @@ class LocationFixer(PageFixer):
                     cities = list(filter(
                         lambda x: 'wikidataid' in x.country.__dict__ and x.country.wikidataid in countryids,
                         cities))
-                if len(cities) == 1:
+                if len(cities) >0:
                     # city can be identified (Could still be incorrect)
-                    final_city=cities[0]
+                    if len(cities) > 1:
+                        errors["city_unclear"] = f"City '{event_city}' matches against multiple cities in the LocationCorpus. Other location information are not sufficient enough to clearly identify the city"
+                        cities=sorted(cities,key=lambda city:0 if 'population'not in city.__dict__ or city.population is None else int(city.population), reverse=True)
+                    final_city = cities[0]
                     if isinstance(final_city,City):
                         event[self.CITY]=self.getPageTitle(final_city)
                         event[self.REGION] = self.getPageTitle(final_city.region)
                         event[self.COUNTRY] = self.getPageTitle(final_city.country)
                         errors["complete"]="Location of event complete"
                         return event, errors
-                elif len(cities) == 0:
+                else:
                     # No matching city -> Two possibilities: event location information incorrect or locations missing in LocationCorpus
                     errors["city_unknown"]=f"City '{event_city}' could not be matched against a city in the LocationCorpus with the given region and country. Either location information are incorrect or location is missing in the corpus."
                     #return None, errors
-                else:
-                    errors["city_unclear"]=f"City '{event_city}' matches against multiple cities in the LocationCorpus. Other location information are not sufficient enough to clearly identify the city"
             else:
                 # event_city is not the LocationCorpus
                 isPossiblyMisplaced=True
@@ -238,12 +222,12 @@ class LocationFixer(PageFixer):
             if possible_cities and possible_regions:
                 # country could be city or region -> undecidable
                 pass
-            elif possible_cities and not cities:
+            elif (possible_cities and not cities) or (possible_cities and modified_country):
                 event[self.CITY]=event_country
                 modified_city=True
                 if not modified_country:
                     del event[self.COUNTRY]
-            elif possible_regions and not regions:
+            elif (possible_regions and not regions) or (possible_cities and modified_country):
                 event[self.REGION] = event_country
                 modified_region=True
                 if not modified_country:
@@ -308,4 +292,95 @@ class LocationFixer(PageFixer):
                 pageTitle=location.__dict__.get('iso')
         else:
             pageTitle=location.name
-        return pageTitle
\ No newline at end of file
+        return pageTitle
+
+
+    def rate(self, rating:EntityRating):
+        '''
+        get the pain Rating for the given eventRecord
+        '''
+        eventRecord=rating.getRecord()
+        arating=self.getRating(eventRecord)
+        rating.set(arating.pain, arating.reason, arating.hint)
+
+    @classmethod
+    def getRating(cls, eventRecord):
+        painrating = None
+        city = None
+        region = None
+        country = None
+
+        if cls.CITY in eventRecord: city = eventRecord[cls.CITY]
+        if cls.REGION in eventRecord: region = eventRecord[cls.REGION]
+        if cls.COUNTRY in eventRecord: country = eventRecord[cls.COUNTRY]
+        if not city and not region and not country:
+            # location is not defined
+            painrating=Rating(7, RatingType.missing,f'Locations are not defined')
+        else:
+            if 'locationContext' in cls.__dict__:
+                cities=cls.__dict__['locationContext'].getCities(city)
+                regions=cls.__dict__['locationContext'].getCities(region)
+                countries=cls.__dict__['locationContext'].getCities(country)
+            if cities and regions and countries:
+                # all locations are recognized
+                painrating=Rating(1,RatingType.ok,f'Locations are valid. (Country: {country}, Region: {region}, City:{city})')
+            elif not cities:
+                # City is not valid
+                painrating=Rating(6, RatingType.invalid,f'City is not recognized. (City:{city})')
+            elif not regions:
+                # City is valid but region is not
+                painrating=Rating(5, RatingType.invalid,f'Region is not recognized. (Country: {country}, Region: {region}, City:{city})')
+            else:
+                # City and region are valid but country is not
+                painrating=Rating(3, RatingType.invalid,f'Country is not recognized. (Country: {country}, Region: {region}, City:{city})')
+        return painrating
+
+    @staticmethod
+    def getORLocationContext():
+        '''
+        Returns a LocationContext enhanced with location labels used by OPENRESEARCH
+        '''
+        locationContext = LocationContext.fromJSONBackup()
+        if locationContext is None:
+            return
+        for locations in locationContext.countries, locationContext.regions, locationContext.cities:
+            for location in locations:
+                LocationFixer._addPageTitleToLabels((location))
+                LocationFixer._addCommonNamesToLabels((location))
+        return locationContext
+
+    @staticmethod
+    def _addPageTitleToLabels(location: Location):
+        '''
+        Adds the pageTitle of the given location to the labels of the given location
+        '''
+        pageTitle = LocationFixer.getPageTitle(location)
+        LocationFixer._addLabelToLocation(location, pageTitle)
+
+    @staticmethod
+    def _addCommonNamesToLabels(location: Location):
+        '''
+        Add commonly used names to the labels
+        '''
+        if isinstance(location, Country):
+            label = f"Category:{location.name}"
+            LocationFixer._addLabelToLocation(location, label)
+
+    @staticmethod
+    def _addLabelToLocation(location: Location, *label: str):
+        '''
+        Adds the given labels to the given location
+        '''
+        for l in label:
+            if 'labels' in location.__dict__:
+                labels = location.__dict__['labels']
+                if isinstance(labels, list):
+                    labels.append(l)
+                else:
+                    labels = [l, labels]
+                location.__dict__['labels'] = labels
+            else:
+                location.__dict__['labels'] = [l]
+    
+if __name__ == '__main__':
+    PageFixerManager.runCmdLine([LocationFixer])
\ No newline at end of file
